"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RoomPresenceDistanceSensor = exports.STATE_NOT_HOME = exports.STATE_HOME = void 0;
const sensor_1 = require("../../entities/sensor");
exports.STATE_HOME = 'home';
exports.STATE_NOT_HOME = 'not_home';
class TimedDistance {
    constructor(distance, outOfRange = false) {
        this.lastUpdatedAt = new Date();
        this.distance = distance;
        this.outOfRange = outOfRange;
    }
}
class RoomPresenceDistanceSensor extends sensor_1.Sensor {
    constructor(id, name, timeout) {
        super(id, name, true);
        this.distances = {};
        this.timeout = timeout;
    }
    handleNewDistance(instanceName, distance, outOfRange = false) {
        this.distances[instanceName] = new TimedDistance(distance, outOfRange);
        this.updateState();
    }
    updateState() {
        const closestInRange = this.getClosestInRange();
        if (closestInRange) {
            if (this.state !== closestInRange[0]) {
                this.state = closestInRange[0];
            }
            if (this.state === closestInRange[0]) {
                this.attributes.distance = closestInRange[1].distance;
                this.attributes.lastUpdatedAt = closestInRange[1].lastUpdatedAt.toISOString();
            }
        }
        else if (this.state !== exports.STATE_NOT_HOME) {
            this.setNotHome();
        }
    }
    getClosestInRange() {
        const distances = Array.from(Object.entries(this.distances))
            .filter((value) => {
            return (!value[1].outOfRange &&
                (this.timeout <= 0 ||
                    Date.now() < value[1].lastUpdatedAt.getTime() + this.timeout * 1000));
        })
            .sort((a, b) => {
            return a[1].distance - b[1].distance;
        });
        return distances.length > 0 ? distances[0] : undefined;
    }
    setNotHome() {
        this.state = exports.STATE_NOT_HOME;
        this.attributes.distance = undefined;
        this.attributes.lastUpdatedAt = new Date().toISOString();
    }
}
exports.RoomPresenceDistanceSensor = RoomPresenceDistanceSensor;
//# sourceMappingURL=room-presence-distance.sensor.js.map