"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThermopileOccupancyService = void 0;
const pixel_1 = require("./pixel");
const math = __importStar(require("mathjs"));
const cluster_1 = require("./cluster");
const _ = __importStar(require("lodash"));
const thermopile_occupancy_config_1 = require("./thermopile-occupancy.config");
const _2d_array_rotation_1 = require("2d-array-rotation");
let nodeCanvas;
try {
    nodeCanvas = require('canvas');
}
catch (e) {
    nodeCanvas = undefined;
}
class ThermopileOccupancyService {
    async getCoordinates(temperatures, deltaThreshold) {
        const relevantPixels = this.findRelevantPixels(temperatures, deltaThreshold);
        const clusters = this.clusterPixels(relevantPixels);
        return clusters.map((cluster) => [cluster.center.x, cluster.center.y]);
    }
    findRelevantPixels(data, deltaThreshold) {
        const mean = math.mean(data);
        const threshold = mean + deltaThreshold;
        const relevantPixels = [];
        for (const [y, row] of data.entries()) {
            for (const [x, value] of row.entries()) {
                if (value >= threshold) {
                    relevantPixels.push(new pixel_1.Pixel(x, y, value));
                }
            }
        }
        return relevantPixels;
    }
    clusterPixels(pixels) {
        const clusters = [];
        pixels.forEach((pixel) => {
            const neighbor = clusters.find((cluster) => cluster.isNeighboredTo(pixel));
            if (neighbor === undefined) {
                clusters.push(new cluster_1.Cluster([pixel]));
            }
            else {
                neighbor.pixels.push(pixel);
            }
        });
        return clusters;
    }
    isHeatmapAvailable() {
        return nodeCanvas !== undefined;
    }
    async generateHeatmap(temperatures, options = new thermopile_occupancy_config_1.HeatmapOptions(), width = 280, height = 280) {
        if (!this.isHeatmapAvailable()) {
            throw new Error('Generating a heatmap requires the canvas optional dependency');
        }
        const segmentHeight = Math.round(height / temperatures.length);
        const segmentWidth = Math.round(width / temperatures[0].length);
        const canvas = [90, 270].includes(options.rotation)
            ? nodeCanvas.createCanvas(height, width)
            : nodeCanvas.createCanvas(width, height);
        const ctx = canvas.getContext('2d');
        let normed = math.divide(math.subtract(temperatures, options.minTemperature), options.maxTemperature - options.minTemperature);
        normed = (0, _2d_array_rotation_1.rotate)(normed, options.rotation);
        const rotatedTemperatures = (0, _2d_array_rotation_1.rotate)(temperatures, options.rotation);
        for (const [y, row] of normed.entries()) {
            for (const [x, value] of row.entries()) {
                const pixelXOrigin = x * segmentWidth;
                const pixelXCenter = pixelXOrigin + segmentWidth / 2;
                const pixelYOrigin = y * segmentHeight;
                const pixelYCenter = pixelYOrigin + segmentHeight / 2;
                ThermopileOccupancyService.drawPixel(ctx, pixelXOrigin, pixelYOrigin, segmentWidth, segmentHeight, value);
                if (options.drawTemperatures) {
                    ThermopileOccupancyService.drawTemperature(ctx, pixelXCenter, pixelYCenter, segmentWidth, rotatedTemperatures[y][x]);
                }
            }
        }
        return canvas.toBuffer('image/png');
    }
    static drawPixel(ctx, xOrigin, yOrigin, width, height, normedValue) {
        const h = (1 - _.clamp(normedValue, 0, 1)) * 240;
        ctx.fillStyle = `hsl(${h}, 100%, 50%)`;
        ctx.fillRect(xOrigin, yOrigin, width, height);
    }
    static drawTemperature(ctx, xCenter, yCenter, width, temperature) {
        ctx.save();
        ctx.font = `${Math.round(width / 2.7)}px sans-serif`;
        ctx.textAlign = 'center';
        ctx.textBaseline = 'middle';
        ctx.fillStyle = 'black';
        ctx.translate(xCenter, yCenter);
        ctx.fillText(temperature.toFixed(1), 0, 0);
        ctx.restore();
    }
}
exports.ThermopileOccupancyService = ThermopileOccupancyService;
//# sourceMappingURL=thermopile-occupancy.service.js.map