"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var GpioService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.GpioService = void 0;
const common_1 = require("@nestjs/common");
const entities_service_1 = require("../../entities/entities.service");
const config_service_1 = require("../../config/config.service");
const onoff_1 = require("onoff");
const binary_sensor_1 = require("../../entities/binary-sensor");
const id_1 = require("../../util/id");
const binary_sensor_config_1 = require("../home-assistant/binary-sensor-config");
const switch_config_1 = require("../home-assistant/switch-config");
const gpio_switch_1 = require("./gpio.switch");
let GpioService = GpioService_1 = class GpioService {
    constructor(configService, entitiesService) {
        this.configService = configService;
        this.entitiesService = entitiesService;
        this.gpios = [];
        this.config = this.configService.get('gpio');
        this.logger = new common_1.Logger(GpioService_1.name);
    }
    onApplicationBootstrap() {
        this.config.binarySensors.forEach((binarySensor) => {
            this.createBinarySensor(binarySensor.name, binarySensor.pin, binarySensor.deviceClass);
        });
        this.config.switches.forEach((switchOptions) => {
            this.createSwitch(switchOptions.name, switchOptions.pin, switchOptions.icon);
        });
    }
    onApplicationShutdown() {
        this.logger.log('Closing opened GPIO pins');
        this.gpios.forEach((gpio) => {
            gpio.unexport();
        });
    }
    createBinarySensor(name, pin, deviceClass) {
        const id = (0, id_1.makeId)(`gpio ${name}`);
        const customizations = [
            {
                for: binary_sensor_config_1.BinarySensorConfig,
                overrides: {
                    deviceClass: deviceClass,
                },
            },
        ];
        const binarySensor = this.entitiesService.add(new binary_sensor_1.BinarySensor(id, name), customizations);
        this.logger.log(`Opening pin ${pin} as input`);
        const gpio = new onoff_1.Gpio(pin, 'in', 'both');
        this.gpios.push(gpio);
        gpio.watch((err, value) => {
            if (err) {
                this.logger.error(err.message, err.stack);
                return;
            }
            binarySensor.state = Boolean(value);
        });
        return binarySensor;
    }
    createSwitch(name, pin, icon) {
        const id = (0, id_1.makeId)(`gpio ${name}`);
        const customizations = [
            {
                for: switch_config_1.SwitchConfig,
                overrides: {
                    icon,
                },
            },
        ];
        const gpio = new onoff_1.Gpio(pin, 'out');
        this.gpios.push(gpio);
        return this.entitiesService.add(new gpio_switch_1.GpioSwitch(id, name, gpio), customizations);
    }
};
GpioService = GpioService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [config_service_1.ConfigService,
        entities_service_1.EntitiesService])
], GpioService);
exports.GpioService = GpioService;
//# sourceMappingURL=gpio.service.js.map