"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var HomeAssistantService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.HomeAssistantService = void 0;
const common_1 = require("@nestjs/common");
const sensor_1 = require("../../entities/sensor");
const entity_config_1 = require("./entity-config");
const sensor_config_1 = require("./sensor-config");
const _ = __importStar(require("lodash"));
const config_service_1 = require("../../config/config.service");
const async_mqtt_1 = __importDefault(require("async-mqtt"));
const device_1 = require("./device");
const systeminformation_1 = require("systeminformation");
const nest_emitter_1 = require("nest-emitter");
const id_1 = require("../../util/id");
const home_assistant_const_1 = require("./home-assistant.const");
const binary_sensor_1 = require("../../entities/binary-sensor");
const binary_sensor_config_1 = require("./binary-sensor-config");
const switch_1 = require("../../entities/switch");
const switch_config_1 = require("./switch-config");
const device_tracker_1 = require("../../entities/device-tracker");
const device_tracker_config_1 = require("./device-tracker-config");
const camera_1 = require("../../entities/camera");
const camera_config_1 = require("./camera-config");
const entities_service_1 = require("../../entities/entities.service");
const cluster_service_1 = require("../../cluster/cluster.service");
const schedule_1 = require("@nestjs/schedule");
const INSTANCE_STATUS_BASE_TOPIC = 'room-assistant/status';
let HomeAssistantService = HomeAssistantService_1 = class HomeAssistantService {
    constructor(configService, entitiesService, clusterService, emitter) {
        this.configService = configService;
        this.entitiesService = entitiesService;
        this.clusterService = clusterService;
        this.emitter = emitter;
        this.entityConfigs = new Map();
        this.debounceFunctions = new Map();
        this.logger = new common_1.Logger(HomeAssistantService_1.name);
        this.config = this.configService.get('homeAssistant');
        const instanceName = this.configService.get('global').instanceName;
        this.statusTopic = `${INSTANCE_STATUS_BASE_TOPIC}/${(0, id_1.makeId)(instanceName)}`;
    }
    async onModuleInit() {
        this.device = await this.getDeviceInfo();
        try {
            this.mqttClient = await async_mqtt_1.default.connectAsync(this.config.mqttUrl, {
                will: {
                    topic: this.statusTopic,
                    payload: 'offline',
                    retain: false,
                    qos: 1,
                    properties: {
                        willDelayInterval: 60,
                    },
                },
                ...this.config.mqttOptions,
            }, false);
            this.sendInstanceStatus();
            this.mqttClient.on('message', this.handleIncomingMessage.bind(this));
            this.mqttClient.on('error', (e) => this.logger.error(e.message, e.stack));
            this.mqttClient.on('connect', this.handleReconnect.bind(this));
            this.logger.log(`Successfully connected to MQTT broker at ${this.config.mqttUrl}`);
            this.emitter.on('newEntity', this.handleNewEntity.bind(this));
            this.emitter.on('entityUpdate', this.handleEntityUpdate.bind(this));
        }
        catch (e) {
            this.logger.error(e.message, e.stack);
        }
        this.clusterService.on('elected', this.refreshEntities.bind(this));
        this.clusterService.on('elected', this.updateDistributedAvailability.bind(this));
        this.clusterService.on('leader', this.updateDistributedAvailability.bind(this));
    }
    async onApplicationShutdown() {
        var _a;
        await this.sendInstanceStatus('offline');
        const entityStatusUpdates = Array.from(this.entityConfigs.values())
            .filter((config) => (config.distributed && this.clusterService.isLeader()) ||
            !config.distributed)
            .map((config) => this.sendEntityStatus(config, 'offline'));
        await Promise.all(entityStatusUpdates);
        await ((_a = this.mqttClient) === null || _a === void 0 ? void 0 : _a.end());
    }
    handleNewEntity(entity, customizations = []) {
        const combinedId = this.getCombinedId(entity.id, entity.distributed);
        let config = this.generateEntityConfig(combinedId, entity);
        if (config === undefined) {
            this.logger.warn(`${combinedId} cannot be matched to a Home Assistant type and will not be transferred`);
            return;
        }
        if (entity.distributed) {
            config.device = new device_1.Device(home_assistant_const_1.DISTRIBUTED_DEVICE_ID);
            config.device.name = 'room-assistant hub';
        }
        else {
            config.device = this.device;
        }
        config = this.applyCustomizations(config, customizations);
        this.entityConfigs.set(combinedId, config);
        this.sendDiscoveryMessage(config);
        if (config.availability.length > 0) {
            this.clearRetained(config.availability[0].topic);
            this.sendEntityStatus(config);
        }
    }
    handleEntityUpdate(entity, diff, hasAuthority) {
        const config = this.entityConfigs.get(this.getCombinedId(entity.id, entity.distributed));
        if (config === undefined) {
            return;
        }
        if (hasAuthority) {
            if (diff.some((diff) => diff.path.startsWith('/state'))) {
                this.sendNewState(entity, config);
            }
            if (this.config.sendAttributes &&
                diff.some((diff) => diff.path.startsWith('/attributes'))) {
                this.sendNewAttributes(entity, config);
            }
            if (this.config.sendMqttRoom &&
                diff.some((diff) => diff.path.startsWith('/distances/'))) {
                this.sendNewMqttRoomDistances(entity.id, entity.name, diff);
            }
        }
    }
    sendHeartbeats() {
        this.sendInstanceStatus();
        this.entityConfigs.forEach((config) => {
            this.sendEntityStatus(config);
        });
    }
    refreshEntities() {
        this.logger.log('Refreshing entity states');
        this.entitiesService.getAll().forEach((entity) => {
            const config = this.entityConfigs.get(this.getCombinedId(entity.id, entity.distributed));
            if (config === undefined) {
                return;
            }
            if (this.entitiesService.hasAuthorityOver(entity)) {
                this.sendNewState(entity, config);
                if (this.config.sendAttributes) {
                    this.sendNewAttributes(entity, config);
                }
            }
        });
    }
    handleIncomingMessage(topic, message) {
        const configs = Array.from(this.entityConfigs.values());
        const config = configs.find((config) => config instanceof switch_config_1.SwitchConfig && config.commandTopic == topic);
        this.logger.debug(`Received message ${message.toString()} on ${topic} for ${config === null || config === void 0 ? void 0 : config.uniqueId}`);
        if (config instanceof switch_config_1.SwitchConfig) {
            const command = message.toString();
            if (config.commandStore[command]) {
                config.commandStore[command]();
            }
        }
    }
    isConnected() {
        var _a;
        return (_a = this.mqttClient) === null || _a === void 0 ? void 0 : _a.connected;
    }
    handleReconnect() {
        this.logger.log('Re-connected to broker');
        this.sendHeartbeats();
        this.refreshEntities();
    }
    async getDeviceInfo() {
        const instanceName = this.configService.get('global').instanceName;
        const systemInfo = await (0, systeminformation_1.system)();
        const serial = systemInfo.serial && systemInfo.serial !== '-'
            ? systemInfo.serial
            : (0, id_1.makeId)(instanceName);
        const device = new device_1.Device(serial);
        device.name = instanceName;
        device.model = systemInfo.model;
        device.manufacturer = systemInfo.manufacturer;
        return device;
    }
    getCombinedId(entityId, distributed = false) {
        return (0, id_1.makeId)(distributed
            ? entityId
            : [this.configService.get('global').instanceName, entityId].join('-'));
    }
    generateEntityConfig(combinedId, entity) {
        var _a;
        let config;
        if (entity instanceof sensor_1.Sensor) {
            config = new sensor_config_1.SensorConfig(combinedId, entity.name);
        }
        else if (entity instanceof binary_sensor_1.BinarySensor) {
            config = new binary_sensor_config_1.BinarySensorConfig(combinedId, entity.name);
        }
        else if (entity instanceof switch_1.Switch) {
            config = new switch_config_1.SwitchConfig(combinedId, entity.name, entity.turnOn.bind(entity), entity.turnOff.bind(entity));
            this.mqttClient.subscribe(config.commandTopic, {
                qos: 0,
            });
        }
        else if (entity instanceof camera_1.Camera) {
            config = new camera_config_1.CameraConfig(combinedId, entity.name);
        }
        else if (entity instanceof device_tracker_1.DeviceTracker) {
            config = new device_tracker_config_1.DeviceTrackerConfig(combinedId, entity.name);
        }
        else {
            return;
        }
        config.distributed = entity.distributed;
        config.stateLocked = entity.stateLocked;
        const instanceId = (0, id_1.makeId)(this.configService.get('global').instanceName);
        const leaderId = entity.distributed
            ? (_a = this.clusterService.leader()) === null || _a === void 0 ? void 0 : _a.id
            : instanceId;
        config.setInstanceStatusTopic(`${INSTANCE_STATUS_BASE_TOPIC}/${leaderId}`);
        return config;
    }
    updateDistributedAvailability(node) {
        Array.from(this.entityConfigs.values())
            .filter((config) => config.distributed && config.stateLocked)
            .forEach(async (config) => {
            config.setInstanceStatusTopic(`${INSTANCE_STATUS_BASE_TOPIC}/${node.id}`);
            if (this.clusterService.isLeader()) {
                await this.sendDiscoveryMessage(config);
                this.sendHeartbeats();
            }
        });
    }
    applyCustomizations(config, customizations) {
        const customization = customizations.find((value) => value.for.prototype instanceof entity_config_1.EntityConfig);
        if (customization !== undefined) {
            Object.assign(config, customization.overrides);
        }
        return config;
    }
    formatMessage(message) {
        const filteredMessage = _.omit(message, entity_config_1.PROPERTY_DENYLIST);
        return this.deepMap(filteredMessage, (obj) => {
            if (!_.isObject(obj) || _.isArray(obj)) {
                return obj;
            }
            return _.mapKeys(obj, (v, k) => {
                return _.snakeCase(k);
            });
        });
    }
    async clearRetained(topic) {
        await this.mqttClient.publish(topic, '', { retain: true });
    }
    async sendEntityStatus(config, status = 'online') {
        var _a;
        if (config.distributed && !config.stateLocked) {
            return;
        }
        this.logger.debug(`Marking ${config.uniqueId} as ${status}`);
        await ((_a = this.mqttClient) === null || _a === void 0 ? void 0 : _a.publish(config.availability[0].topic, status));
    }
    async sendInstanceStatus(status = 'online') {
        var _a;
        this.logger.debug(`Marking instance as ${status}`);
        await ((_a = this.mqttClient) === null || _a === void 0 ? void 0 : _a.publish(this.statusTopic, status, {
            qos: 1,
        }));
    }
    async sendDiscoveryMessage(config) {
        var _a;
        if (config.distributed &&
            config.stateLocked &&
            !this.clusterService.isLeader()) {
            return;
        }
        const message = this.formatMessage(config instanceof camera_config_1.CameraConfig
            ? _.omit(config.shallowClone(), ['stateTopic'])
            : config.shallowClone());
        const discoveryTopic = this.config.discoveryPrefix + '/' + config.configTopic;
        this.logger.debug(`Registering entity ${config.uniqueId} under ${discoveryTopic}`);
        await ((_a = this.mqttClient) === null || _a === void 0 ? void 0 : _a.publish(discoveryTopic, JSON.stringify(message), {
            qos: 0,
            retain: true,
        }));
    }
    sendNewState(entity, config) {
        const logState = entity.state instanceof Buffer ? '<binary>' : entity.state;
        this.logger.debug(`Sending new state ${logState} for ${config.uniqueId}`);
        this.mqttClient.publish(config.stateTopic, entity.state instanceof Buffer ? entity.state : String(entity.state), {
            qos: 0,
            retain: true,
        });
    }
    sendNewAttributes(entity, config) {
        if (this.debounceFunctions.has(entity.id)) {
            this.debounceFunctions.get(entity.id)(entity.attributes);
        }
        else {
            const debouncedFunc = _.debounce((attributes) => {
                this.logger.debug(`Sending new attributes ${JSON.stringify(attributes)} for ${config.uniqueId}`);
                this.mqttClient.publish(config.jsonAttributesTopic, JSON.stringify(this.formatMessage(attributes)));
            });
            this.debounceFunctions.set(entity.id, debouncedFunc);
            debouncedFunc(entity.attributes);
        }
    }
    sendNewMqttRoomDistances(id, name, diff) {
        diff
            .filter((value) => value.path.startsWith('/distances/'))
            .forEach((value) => {
            const room = value.path.split('/')[2];
            this.mqttClient.publish(`${this.config.mqttRoomPrefix}/${room}`, JSON.stringify({
                id,
                name,
                distance: value.newValue.distance,
            }));
        });
    }
    deepMap(obj, mapper) {
        if (!_.isObject(obj)) {
            return mapper(obj);
        }
        const mappingMethod = _.isArray(obj) ? _.map : _.mapValues;
        return mapper(mappingMethod(obj, (v) => {
            if (_.isArray(v)) {
                return [...v].map((e) => this.deepMap(e, mapper));
            }
            else if (_.isObject(v)) {
                return this.deepMap(v, mapper);
            }
            else {
                return v;
            }
        }));
    }
};
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_30_SECONDS),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], HomeAssistantService.prototype, "sendHeartbeats", null);
HomeAssistantService = HomeAssistantService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(3, (0, nest_emitter_1.InjectEventEmitter)()),
    __metadata("design:paramtypes", [config_service_1.ConfigService,
        entities_service_1.EntitiesService,
        cluster_service_1.ClusterService, Object])
], HomeAssistantService);
exports.HomeAssistantService = HomeAssistantService;
//# sourceMappingURL=home-assistant.service.js.map