"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var XiaomiMiService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.XiaomiMiService = void 0;
const common_1 = require("@nestjs/common");
const entities_service_1 = require("../../entities/entities.service");
const config_service_1 = require("../../config/config.service");
const home_assistant_const_1 = require("../home-assistant/home-assistant.const");
const id_1 = require("../../util/id");
const parser_1 = require("./parser");
const sensor_1 = require("../../entities/sensor");
const sensor_config_1 = require("../home-assistant/sensor-config");
const bluetooth_service_1 = require("../../integration-support/bluetooth/bluetooth.service");
const entity_config_1 = require("../home-assistant/entity-config");
const SERVICE_DATA_UUID = 'fe95';
const SERVICE_BATTERY_UUID = '0000120400001000800000805f9b34fb';
const CHARACTERISTIC_BATTERY_UUID = '00001a0200001000800000805f9b34fb';
const BATTERY_QUERY_WINDOW = 60 * 60 * 1000;
const BATTERY_QUERY_INTERVAL = 23.5 * 60 * 60 * 1000;
const BATTERY_QUERY_ATTEMPTS = 3;
class SensorMetadata {
}
const SensorMetadataList = {
    [4100]: {
        name: 'Temperature',
        deviceClass: 'temperature',
        stateClass: entity_config_1.StateClass.MEASUREMENT,
        units: '°C',
    },
    [4102]: {
        name: 'Humidity',
        deviceClass: 'humidity',
        stateClass: entity_config_1.StateClass.MEASUREMENT,
        units: '%',
    },
    [4106]: {
        name: 'Battery',
        deviceClass: 'battery',
        stateClass: entity_config_1.StateClass.MEASUREMENT,
        units: '%',
    },
    [4103]: {
        name: 'Illuminance',
        deviceClass: 'illuminance',
        stateClass: entity_config_1.StateClass.MEASUREMENT,
        units: 'lx',
    },
    [4104]: {
        name: 'Moisture',
        deviceClass: undefined,
        stateClass: entity_config_1.StateClass.MEASUREMENT,
        units: '%',
    },
    [4105]: {
        name: 'Conductivity',
        deviceClass: undefined,
        stateClass: entity_config_1.StateClass.MEASUREMENT,
        units: 'µS/cm',
    },
};
class BatterySchedule {
}
let XiaomiMiService = XiaomiMiService_1 = class XiaomiMiService {
    constructor(bluetoothService, entitiesService, configService) {
        this.bluetoothService = bluetoothService;
        this.entitiesService = entitiesService;
        this.configService = configService;
        this.lastFrameSeen = [];
        this.batterySchedule = [];
        this.logger = new common_1.Logger(XiaomiMiService_1.name);
    }
    onModuleInit() {
        this.config = this.configService.get('xiaomiMi');
        if (!this.hasSensors()) {
            this.logger.warn('No sensors entries in the config, so no sensors will be created! ' +
                'Enable the bluetooth-low-energy integration to log discovered IDs.');
        }
    }
    onApplicationBootstrap() {
        this.bluetoothService.onLowEnergyDiscovery(this.handleDiscovery.bind(this));
    }
    hasSensors() {
        var _a, _b;
        return ((_b = (_a = this.config) === null || _a === void 0 ? void 0 : _a.sensors) === null || _b === void 0 ? void 0 : _b.length) > 0;
    }
    recordMeasure(device, sensor, state) {
        this.logger.debug(`${device.name}: ${sensor.name}: ${state}${sensor.units}`);
        const id = (0, id_1.makeId)(`xiaomi ${device.identifiers} ${sensor.name}`);
        let entity = this.entitiesService.get(id);
        if (!entity) {
            const customizations = [
                {
                    for: sensor_config_1.SensorConfig,
                    overrides: {
                        deviceClass: sensor.deviceClass,
                        stateClass: sensor.stateClass,
                        unitOfMeasurement: sensor.units,
                        device: device,
                    },
                },
            ];
            entity = this.entitiesService.add(new sensor_1.Sensor(id, `${device.name} ${sensor.name}`, true, false), customizations);
        }
        entity.state = state;
    }
    async handleDiscovery(peripheral) {
        var _a, _b, _c;
        const sensorConfig = (_b = (_a = this.config) === null || _a === void 0 ? void 0 : _a.sensors) === null || _b === void 0 ? void 0 : _b.find((el) => el.address === peripheral.id);
        if (!sensorConfig) {
            return;
        }
        const serviceData = this.parseAdvertisement(peripheral.advertisement, sensorConfig);
        if (!serviceData) {
            return;
        }
        if (this.lastFrameSeen[peripheral.id] == serviceData.frameCounter) {
            return;
        }
        this.lastFrameSeen[peripheral.id] = serviceData.frameCounter;
        const device = {
            name: sensorConfig.name,
            manufacturer: 'Xiaomi',
            model: serviceData.productName,
            identifiers: peripheral.id,
            viaDevice: home_assistant_const_1.DISTRIBUTED_DEVICE_ID,
        };
        serviceData.events.forEach((event) => {
            const metadata = SensorMetadataList[event.type];
            if (metadata) {
                this.recordMeasure(device, metadata, event.value);
            }
            else {
                this.logger.error(`${sensorConfig.name}: unknown event type: ${serviceData.eventType}`);
            }
        });
        const checkBattery = (_c = sensorConfig.enableMifloraBattery) !== null && _c !== void 0 ? _c : this.config.enableMifloraBattery;
        if (serviceData.productId == 152 && checkBattery) {
            await this.checkMifloraBattery(peripheral, device);
        }
    }
    parseAdvertisement(advertisement, sensorConfig) {
        const buffer = XiaomiMiService_1.findServiceData(advertisement, SERVICE_DATA_UUID);
        if (!buffer) {
            this.logger.debug(`${sensorConfig.name}: skipping message as supported data format not present: ${JSON.stringify(advertisement)}`);
            return null;
        }
        let serviceData = null;
        try {
            serviceData = XiaomiMiService_1.parseServiceData(buffer, sensorConfig.bindKey);
        }
        catch (error) {
            this.logger.error(`${sensorConfig.name}: skipping message as received data corrupt: ${error}`);
            return null;
        }
        if (!serviceData.frameControl.hasEvent) {
            this.logger.debug(`${sensorConfig.name}: skipping message as no sensor data provided: ${buffer.toString('hex')}`);
            return null;
        }
        return serviceData;
    }
    static findServiceData(advertisement, suuid) {
        if (!(advertisement === null || advertisement === void 0 ? void 0 : advertisement.serviceData)) {
            return null;
        }
        const uuidPair = advertisement.serviceData.find((data) => data.uuid.toLowerCase() === suuid);
        if (!uuidPair) {
            return null;
        }
        return uuidPair.data;
    }
    static parseServiceData(buffer, bindKey) {
        return new parser_1.Parser(buffer, bindKey).parse();
    }
    async checkMifloraBattery(peripheral, device) {
        if (!this.batterySchedule.hasOwnProperty(peripheral.id)) {
            this.batterySchedule[peripheral.id] = {
                nextQueryAfter: new Date(Date.now() + Math.random() * BATTERY_QUERY_WINDOW),
                attempts: 0,
            };
        }
        const schedule = this.batterySchedule[peripheral.id];
        if (Date.now() > schedule.nextQueryAfter.getTime()) {
            let buffer = null;
            if (!this.bluetoothService.acquireQueryMutex()) {
                this.logger.debug(`${device.name}: Canceled battery reading as BLE adapter is already in use`);
                return;
            }
            schedule.attempts += 1;
            try {
                buffer = await this.bluetoothService.queryLowEnergyDevice(peripheral, SERVICE_BATTERY_UUID, CHARACTERISTIC_BATTERY_UUID);
                if (!buffer) {
                    throw new Error('data unavailable');
                }
            }
            catch (error) {
                this.logger.warn(`${device.name}: Error reading battery level (attempt ${schedule.attempts} of ${BATTERY_QUERY_ATTEMPTS}): ${error}`);
            }
            this.bluetoothService.releaseQueryMutex();
            if (buffer) {
                if (buffer.length > 2) {
                    device.swVersion = buffer.toString('utf-8', 2);
                }
                this.recordMeasure(device, SensorMetadataList[4106], buffer[0]);
                this.logger.log(`${device.name}: Battery level updated to ${buffer[0]}%`);
            }
            if (buffer || schedule.attempts >= BATTERY_QUERY_ATTEMPTS) {
                schedule.nextQueryAfter = new Date(Date.now() +
                    BATTERY_QUERY_INTERVAL +
                    Math.random() * BATTERY_QUERY_WINDOW);
                schedule.attempts = 0;
                if (!buffer) {
                    this.logger.error(`${device.name}: Battery query aborted after reaching maximum number of retries`);
                }
            }
        }
    }
};
XiaomiMiService = XiaomiMiService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [bluetooth_service_1.BluetoothService,
        entities_service_1.EntitiesService,
        config_service_1.ConfigService])
], XiaomiMiService);
exports.XiaomiMiService = XiaomiMiService;
//# sourceMappingURL=xiaomi-mi.service.js.map