"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var HealthIndicatorService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.HealthIndicatorService = void 0;
const terminus_1 = require("@nestjs/terminus");
const common_1 = require("@nestjs/common");
const schedule_1 = require("@nestjs/schedule");
let notify;
try {
    notify = require('sd-notify');
}
catch (e) {
    notify = undefined;
}
let HealthIndicatorService = HealthIndicatorService_1 = class HealthIndicatorService {
    constructor(health, schedulerRegistry) {
        this.health = health;
        this.schedulerRegistry = schedulerRegistry;
        this.healthIndicators = [];
        this.logger = new common_1.Logger(HealthIndicatorService_1.name);
    }
    onApplicationBootstrap() {
        if (notify == undefined) {
            this.logger.debug('Systemd watchdog reporting is unavailable, as the optional sd-notify dependency could not be loaded');
        }
        notify === null || notify === void 0 ? void 0 : notify.ready();
        const watchdogInterval = notify === null || notify === void 0 ? void 0 : notify.watchdogInterval();
        if (watchdogInterval > 0) {
            const interval = Math.floor(watchdogInterval / 2);
            const notifyInterval = setInterval(this.updateWatchdog.bind(this), interval);
            this.schedulerRegistry.addInterval('health-indicator-watchdog', notifyInterval);
        }
    }
    onModuleDestroy() {
        notify === null || notify === void 0 ? void 0 : notify.stopping();
    }
    getIndicators() {
        return this.healthIndicators;
    }
    registerHealthIndicator(healthIndicatorFunction) {
        this.healthIndicators.push(healthIndicatorFunction);
    }
    check() {
        return this.health.check(this.getIndicators());
    }
    async updateWatchdog() {
        const healthResult = await this.check();
        if (healthResult.status != 'error') {
            notify.watchdog();
        }
    }
};
HealthIndicatorService = HealthIndicatorService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [terminus_1.HealthCheckService,
        schedule_1.SchedulerRegistry])
], HealthIndicatorService);
exports.HealthIndicatorService = HealthIndicatorService;
//# sourceMappingURL=health-indicator.service.js.map