"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatusService = void 0;
const common_1 = require("@nestjs/common");
const entities_service_1 = require("../entities/entities.service");
const cluster_service_1 = require("../cluster/cluster.service");
const config_service_1 = require("../config/config.service");
const sensor_1 = require("../entities/sensor");
const sensor_config_1 = require("../integrations/home-assistant/sensor-config");
let StatusService = class StatusService {
    constructor(entitiesService, clusterService, configService) {
        this.entitiesService = entitiesService;
        this.clusterService = clusterService;
        this.configService = configService;
    }
    onApplicationBootstrap() {
        this.clusterSizeSensor = this.createClusterSizeSensor();
        this.clusterLeaderSensor = this.createClusterLeaderSensor();
        this.updateClusterSizeSensor();
        this.updateClusterLeaderSensor();
        this.clusterService.on('added', this.updateClusterSizeSensor.bind(this));
        this.clusterService.on('removed', this.updateClusterSizeSensor.bind(this));
        this.clusterService.on('elected', this.updateClusterLeaderSensor.bind(this));
        this.clusterService.on('leader', this.updateClusterLeaderSensor.bind(this));
    }
    updateClusterSizeSensor() {
        const nodes = Object.keys(this.clusterService.nodes());
        this.clusterSizeSensor.state = nodes.length;
        this.clusterSizeSensor.attributes.nodes = nodes;
    }
    updateClusterLeaderSensor() {
        var _a;
        this.clusterLeaderSensor.state = ((_a = this.clusterService.leader()) === null || _a === void 0 ? void 0 : _a.id) || 'none';
        this.clusterLeaderSensor.attributes.quorumReached = this.clusterService.quorumReached();
    }
    createClusterSizeSensor() {
        const instanceName = this.configService.get('global').instanceName;
        const customizations = [
            {
                for: sensor_config_1.SensorConfig,
                overrides: {
                    icon: 'mdi:server',
                    unitOfMeasurement: 'instances',
                },
            },
        ];
        const clusterSizeSensor = this.entitiesService.add(new sensor_1.Sensor('status-cluster-size', `${instanceName} Cluster Size`), customizations);
        return clusterSizeSensor;
    }
    createClusterLeaderSensor() {
        const instanceName = this.configService.get('global').instanceName;
        const customizations = [
            {
                for: sensor_config_1.SensorConfig,
                overrides: {
                    icon: 'mdi:account-group',
                },
            },
        ];
        const clusterLeaderSensor = this.entitiesService.add(new sensor_1.Sensor('status-cluster-leader', `${instanceName} Cluster Leader`), customizations);
        return clusterLeaderSensor;
    }
};
StatusService = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [entities_service_1.EntitiesService,
        cluster_service_1.ClusterService,
        config_service_1.ConfigService])
], StatusService);
exports.StatusService = StatusService;
//# sourceMappingURL=status.service.js.map